// defined constants for Companion 

const myToolbarName = "Companion"; // 
const myToolbarId = "yahootoolbar"; // <toolbar>'s id
const myMenulistId = "yahootoolbarSearchText";
const ID = '{a3c7a99c-4a06-44bb-b73c-8451bcf6dfd8}'; // match the install.rdf's GUID

// global variables

function yahootoolbar() {
    this.preferences = new Preferences();
    this.myToolbarId = myToolbarId;
    this.myMenulistId = myMenulistId;
    this.isLoggedIn = false;
    this.refreshcnt = 0;        // Init the refresh counter
    this.displayer = new CompanionDisplayer(this.preferences);
    
};


yahootoolbar.prototype.DisplayCheck = function (){
    var main = document.getElementById("main-window");
    var myToolbar = document.getElementById('yahootoolbar_dyn');

    var d = window._content.document;
    // use a regex to match things like uk.search.yahoo.com
    if (d.location && d.location.href.match(/search\.yahoo\.com\//))
    {
        // The search string is the value of the p= parameter in the URL
        // Look for the 'p=' following a '&' or the initial '?'
        var tmp = d.location.search.match(/[&|^?]p=([^&]*)/);
        yahootoolbar.AddSearchTerm(decodeURIComponent(tmp[1]), "yahootoolbarSearchTextDyn");
    }
}

// This will make sure that the toolbar is up to date by redownloading the feed
// and applying it's settings.
yahootoolbar.prototype.dynRefresh = function()
{
    // refresh toolbar
    this.dynOpen();

    // Set the full refresh counter
    this.refreshcnt = this.preferences.refreshCount;
    
    // keep the alert button up to date
    setTimeout("yahootoolbar.refreshAlert()", 60000);
}

// Check for new mail and refresh the alert button
// This is what ensures that the "Notification" feed is grabbed
// and the little alarm bell appears.
yahootoolbar.prototype.refreshAlert = function()
{
         
    
    if (this.refreshcnt-- == 0)    //Time for a full refresh
    {
        this.dynOpen();
        this.refreshcnt = this.preferences.refreshCount;
    } else {
    
        var io=Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    
        var url =  this.preferences.updateServer + "/slv/v4/not";
	var channel = io.newChannel(url, 0, null);
        var parser = new NewMailNotify(this.displayer);
        var streamer = new StreamListener(parser, null);
        channel.asyncOpen(streamer, null);
    }

    setTimeout("yahootoolbar.refreshAlert()", 60000);
}

// dynOpen()
// This is where the whole thing starts.  This method constructs all the main players
// required to dynamically display the toolbar.
// The CompanionDisplayer class will handle the rendering of the objects.
// The FeedParser will parse the Companion feed into an array of internal objects.
// The StreamListener will handle the fetching of the HTTP data from the Yahoo server.
//
// The StreamListener will invoke the FeedParser.parse() method passing all the data it received.
// The FeedParser will parse the data into an Array of objects.  It will then invoke the
// CompanionDisplayer.display() method passing the array of objects.
//
// This design provides a bit of componentization for the yahoo toolbar.  The StreamListener
// is really pretty independent of everything.  The FeedParser understands the Companion
// feed format, but only makes objects.  The CompanionDisplayer doesn't know anything about
// the wacky Yahoo format, just the objects and classes defined here.
yahootoolbar.prototype.dynOpen = function()
{
    
    // from http://bugzilla.mozilla.org/attachment.cgi?id=108723&action=view
    var io=Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);

    var url =  this.preferences.updateServer + "/slv/v4";
    url += "/"+this.preferences.feed+".html" ;
    url += "?.ta=" + this.preferences.feedAttrs ;
    var channel = io.newChannel(url, 0 , null);
    var parser = new FeedParser(this.displayer);
    //parser.debug=true;
    var streamer = new StreamListener(parser, null);
    channel.asyncOpen(streamer, null);
}

yahootoolbar.prototype.onClick = function(evt)
{
    var url = this.InsertSearchText(evt.target.link);
    
    switch (evt.button)
    {
        case 1:
            this.OpenTab(url);
            break;
        case 2: 
            window.open(url);
            break;
     }
     return false;
}

yahootoolbar.prototype.OpenTabs = function(evt)
{
    var abrowser = document.getElementById("content");

    var links = evt.target.parentNode.childNodes;

    for (var i=0; i<links.length; i++){
        var link = links[i].link;
	if (link) 
	    abrowser.addTab(link);
    }

}

yahootoolbar.prototype.OpenTab = function(url)
{
    var prefSvc = Components.classes["@mozilla.org/preferences-service;1"]
                 .getService(Components.interfaces.nsIPrefService);
    prefSvc = prefSvc.getBranch(null);

    var abrowser = document.getElementById("content");
     
    if (prefSvc.getBoolPref("browser.tabs.loadInBackground"))
        abrowser.addTab(url);
    else
        abrowser.selectedTab = abrowser.addTab(url);
            
           
} 
 
yahootoolbar.prototype.LoadPage = function(url, ctrl)
{
    if (!url || url == '') return false;

    var searchUrl = this.InsertSearchText(url);
    if (ctrl)
        this.OpenTab(url);
    else
       window.content.document.location.href=searchUrl;
    return false;
}

// Put the add bookmark function here to avoid 1000 copies of the same script
yahootoolbar.prototype.AddBookmark = function(evt)
{
        var folder = evt.target.link;
	// directly call bookmark api
	var url="http://e.my.yahoo.com/config/add_site?.folder=";
	if (folder)
		url += folder;
                
	// what we actually need here is to make a background request
	var d = window._content.document; 
	var title = d.title;
	if (evt.ctrlKey) {
 
	    var bookmarkPrompt = document.getElementById("yahootoolbar_strings").getString("bookmarkPrompt");
	    title = prompt(bookmarkPrompt, title);
	    if (title == null)
	        return;
	}
	if (this.displayer.charset != "iso-8859-1")
	{
		var uc = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
		uc.charset = this.displayer.charset;
		var bmTitle = uc.ConvertFromUnicode(title);
	} else {
		var bmTitle = title; 
    	}		
    	url += "&.name="+escape(bmTitle)+"&.url="+encodeURIComponent(d.location); 
    	//alert('URL to add is: ' + url);
    	var obj = new XMLHttpRequest();
    	obj.open('GET',url);
	obj.send(null);
            
	// what we need here is a way to refresh the toolbar.
	// but how to do that from javascript?
	// for now, just tell the user we added it
	// setTimeout();
                 
	// Set the countdown timer to zero so the next refresh will refresh all
	this.refreshcnt = 0;
	//alert('Added Bookmark.  Refresh Toolbar to see the link.');
	
	var str = document.getElementById("yahootoolbar_strings").getString("bookmarkAlert");
	alert(str);
               
}

// Highlight the current search words
yahootoolbar.prototype.toggleHighlight = function(e)
{
	var tmp = e.target;
}


// Insert the users search query by replacing %10EditBox
yahootoolbar.prototype.InsertSearchText = function(url)
{
    // get the menulist element
    var mySearchBox = document.getElementById('yahootoolbarSearchTextDyn');
    
    // extract its value
    var mySearchTerm = mySearchBox.value;
  
    // insert search text into url
    var re = /.EditBox(\x01%!)?/g;
    var searchUrl = url.replace(re, mySearchTerm);
    var d = window._content.document;
    re    = /\x01\$GETTITLE/g;
    searchUrl = searchUrl.replace(re,encodeURIComponent(d.title));
    re    = /\x01\$GETURL/g;
    searchUrl = searchUrl.replace(re,encodeURIComponent(d.location));
    re    = /\x01\$ESRV/g;
    searchUrl = searchUrl.replace(re,"edit.companion.yahoo.com");


    // insert domain into into search
    try {
        var reYahooDomain = /\x01\$DOMAIN/g;
        var reDomain = new RegExp(/\/\/([a-zA-Z0-9\.]*)/);
        var currentAddress = getWebNavigation().currentURI.spec;
        var m = reDomain.exec(currentAddress);
        currentAddress = currentAddress.match(m[1]);
        var siteSearchUrl = searchUrl.replace(reYahooDomain, currentAddress);
        searchUrl = siteSearchUrl;
    } catch (e) {
    }
    
    // The Companion team from Yahoo asked that we add this
    // identified to help track how many searches come from our toolbar
    
    // borrowing from http://www.felgall.com/javadom6.htm
    var agt = navigator.userAgent.toLowerCase();
    var pszBrowserSearchId;
    if(agt.indexOf("firefox")!= -1)
    {
        pszBrowserSearchId = "ff";
    }
    else if(agt.indexOf("netscape") != -1)
    {
        pszBrowserSearchId = "nscp";
    }
    else
    {
        pszBrowserSearchId = "moz";
    }
    
    // finally, clean up url
    re = /\x01\$REG_sc/g;  
    searchUrl = searchUrl.replace(re, pszBrowserSearchId);
  
    re = /\x01\%/g;  
    searchUrl = searchUrl.replace(re, "");

    return searchUrl;
}

// the observe method is called whenever a "cookieChanged" event happens
// which (not suprisingly) happens whenever any cookie is modified, added,
// or removed.
yahootoolbar.prototype.observe = function(subject,topic,state)
{
    if(topic != "cookieChanged" && topic != 'cookie-changed')
        return;

    // see http://lxr.mozilla.org/seamonkey/source/extensions/wallet/cookieviewer/CookieViewer.js
    // for reference
    // the basic idea here is to iterate thru all the cookies,
    // and look for yahoo cookies.  From testing (using CookieManager),
    // you can see that a Y and T cookie will be set when a browser logs into Yahoo.
    var log = "";
    var cookiemanager = Components.classes["@mozilla.org/cookiemanager;1"].getService();
    cookiemanager = cookiemanager.QueryInterface(Components.interfaces.nsICookieManager);
    var cookieEnum = cookiemanager.enumerator;
    var has_y = 0;
    var has_t = 0;
    log += "Cookies: \n";
    while(cookieEnum.hasMoreElements())
    {
        var nextCookie = cookieEnum.getNext();
        if (!nextCookie) 
            break;
        nextCookie = nextCookie.QueryInterface(Components.interfaces.nsICookie);
        log += "host: " + nextCookie.host + " -- name: " + nextCookie.name + "\n";
        if(nextCookie.host.indexOf("yahoo.com") != -1)
        {
           if(nextCookie.name == "Y")
               has_y = 1;
           else if(nextCookie.name == "T")
               has_t = 1;
        }
    }
    
    log += "Has Y : " + has_y + " Has T: " + has_t + "\n";
    
    if(has_y == 1 && has_t == 1)
    {
        // the browser is currently logged in
        // if the toolbar is NOT LOGGED IN, then we should refresh
        if(this.isLoggedIn == false)
        {
            this.isLoggedIn = true;
            this.dynOpen();
        }
    }
    else
    {
        // the browser is currently logged out
        // if the toolbar is LOGGED IN, then we should refresh
        if(this.isLoggedIn == true)
        {
            this.isLoggedIn = false;
            this.dynOpen();
        }
    }
    // this is for debugging purposes.
    //this.save("cookies-log", log);
}

// save is a debugging routine that will save any data the second argument
// to a file named fname in the chrome/yahootoolbar_saves/ dir.
yahootoolbar.prototype.save = function(fname, txt)
{
    var aFile = Components.classes["@mozilla.org/file/local;1"].createInstance();
    if (!aFile)
        return false;
    var aLocalFile=aFile.QueryInterface(Components.interfaces.nsILocalFile);
    if (!aLocalFile)
        return false;
    
    // where to save the data ?
    var dirServ = Components.classes['@mozilla.org/file/directory_service;1'].createInstance();
    dirServ = dirServ.QueryInterface(Components.interfaces.nsIProperties);
    var processDir = dirServ.get("AChrom", Components.interfaces.nsIFile);
    // on a windows machine, processDir might be C:\Program Files\mozilla.org\Mozilla\chrome
    processDir.append("yahootoolbar_saves");
    if(! processDir.exists())
        processDir.create(processDir.DIRECTORY_TYPE, 0766);
    
    // now we init the LocalFile with the dir we just made
    processDir.append(fname);
    aLocalFile.initWithFile(processDir);

    // make the file if its not there.
    if(!aLocalFile.exists())
        aLocalFile.create(aLocalFile.NORMAL_FILE_TYPE, 0766);
        
    // now we write out the text.
    var stream = Components.classes['@mozilla.org/network/file-output-stream;1'].createInstance(Components.interfaces.nsIFileOutputStream);
    stream.init(aLocalFile, 2, 0666, false); // open as "write only"
    stream.write(txt, txt.length);
    stream.close();
}


// registerListener is called to tell the main Mozilla app that
// we are interested in observing the cookieChanged event, and that
// the 'this' object will handle it.  Note that the 'this.observe()'
// method will be called.
yahootoolbar.prototype.registerListener = function()
{
    // http://lxr.mozilla.org/seamonkey/source/extensions/wallet/cookieviewer/CookieViewer.js#123
    try
    {
        yahooToolbarObserverService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        if(yahooToolbarObserverService != null)
        {
            yahooToolbarObserverService.addObserver(this, "cookieChanged", false);
            yahooToolbarObserverService.addObserver(this, "cookie-changed", false);
        }
        //else
        //    alert("Can't register observer with observer service!");
    }
    catch(e)
    {
        //alert("failed to get observer service");
    }
}

// dynSearch just performs the search.  It will
// use the passed url as the base.  Since some of the
// yahoo search uses unusual query strings, the token
// 'EditBox' (optionally surrounded by \x01 chars) will
// be replaced from the provided url.
yahootoolbar.prototype.dynSearch = function(url)
{
    // get the menulist element
    var mySearchBox = document.getElementById('yahootoolbarSearchTextDyn');
  
    // extract its value
    var mySearchTerm = encodeURIComponent(mySearchBox.value);
    
    //yahootoolbar.AddSearchTerm(mySearchTerm, "yahootoolbarSearchTextDyn");
    
    if(url == null)
    {
        url = this.preferences.searchURL || 
              "http://search.yahoo.com/bin/search?ei=UTF-8&fr=slv1-\x01$REG_sc&p=\x01EditBox";
    }

    var searchTermsMarker = "<searchTerms>";
    var searchUrl = url.replace(searchTermsMarker, mySearchTerm);
    //var idx = url.indexOf(searchTermsMarker);
    //alert("Original url: " + url+ "\nNew url: " + searchUrl + "\nIndex of ^AEditBox: " + idx);
    this.LoadPage(searchUrl);
}

//add a search term to the drop down.
yahootoolbar.prototype.AddSearchTerm = function(term, menuid){
    var menuList = document.getElementById(menuid);
  
    //alert('fetched search menu list from id ' + menuid + ' and got: ' + mySearchMenuList);
    
    // lets check the length of the drop down.  Since we have the <menulist>, 
    // we go to its first child (<menupopup>), and get the number of its children
    // (<menuitem>'s)
    // If there are more than 6 children, we remove the last one.
    if (menuList.firstChild)
    {
        var menuItem = menuList.firstChild.firstChild;
        while (menuItem)
        {
            if (menuItem.label == term) 
                menuList.firstChild.removeChild(menuItem);
            menuItem = menuItem.nextSibling;
        }
      
        if (menuList.firstChild && menuList.firstChild.childNodes.length > 6)
            menuList.removeItemAt(6);
    }  
    menuList.insertItemAt(0,term);  
}

// this method retrieves the version number
// from the install.rdf file.  This code is based on the ExtensionManager
// code.. the following links really helped:
// http://lxr.mozilla.org/aviarybranch/source/toolkit/mozapps/extensions/public/nsIExtensionManager.idl
// http://lxr.mozilla.org/aviarybranch/source/toolkit/mozapps/extensions/content/extensions.xml
// http://lxr.mozilla.org/aviarybranch/source/toolkit/mozapps/extensions/content/extensions.js
// http://lxr.mozilla.org/aviarybranch/source/toolkit/mozapps/extensions/content/extensions.xul
// http://lxr.mozilla.org/aviarybranch/source/toolkit/mozapps/update/public/nsIUpdateService.idl#45
// http://www98.sakura.ne.jp/~piro/xul/codes/policymanager/content/policymanager/about.xul
yahootoolbar.prototype.getVersion = function () {
  
    /* this will work for firefox but will fail for mozilla */
    try
    {
        var cExtMan = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
        // try to grab our info
        var cnt = Object();
	if (typeof(cExtMan.getItemForID) == 'function') {
	    var item = cExtMan.getItemForID(ID);
		return item.version;
	} else {
            var item = cExtMan.getItemList(ID,Components.interfaces.nsIUpdateItem.TYPE_EXTENSION, cnt);
            if(item && item[0])
                return item[0].version;
            else
                return 0;
	}
    }
    catch(e)
    {
	// mozilla test
	var iVersion = InstallTrigger.getVersion("/mozilla/yahootoolbar");
	if(iVersion != null)
	    return iVersion;
	else
	    return -1;
    }
}

// showAbout simply popups open the little dialog box
// which says who we are, what version, etc.
yahootoolbar.prototype.showAbout = function () {
    var msgbox = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    var ver = this.getVersion();
    var myStrings = document.getElementById("yahootoolbar_strings");
    var myProduct = myStrings.getString("toolbar");
    var abouttxt = myStrings.getFormattedString("toolbarAbout",[myProduct, ver]);
    msgbox.alert(window,myProduct,abouttxt);
     //        "Yahoo! Companion (Mozilla)\n"
     //        +"Version: "+ver+"\n"
     //        +"http://companion.mozdev.org");
}

// Uninstall ******* 
yahootoolbar.prototype.Uninstall = function(url){
    netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect"); 
    var uninstall = document.getElementById("yahootoolbar_strings").
                    getFormattedString("uninstall", [myToolbarName]);
    //var confirmUninstall = confirm("Do you want to uninstall " + myToolbarName+ " ?" );
    var confirmUninstall = confirm(uninstall);
    
    if (!confirmUninstall) // user choose to cancel
        return;
    
    // remove the toolbar
    //var myToolBar = document.getElementById(myToolbarId);
    var myToolBar = document.getElementById('yahootoolbar_dyn');
    myToolBar.parentNode.removeChild(myToolBar);

    // remove the yahoo bookmark browser menu
    this.displayer.removeYahooBrowserBookmarksMenu();

    // If Firefox, use the extension manager
    try
    {
       var cExtMan = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
       cExtMan.uninstallExtension(ID);
       return;
    } catch(e)
    {
    	// No extension manager - no problem.
    }
    
    // remove the view menu entry
    var myViewMenuEntry = document.getElementById("view_"+myToolbarId+"_menuitem");
    if (myViewMenuEntry)
    	myViewMenuEntry.parentNode.removeChild(myViewMenuEntry);
  
    //remove the overlay entry in overlays.rdf
  
    var CONTAINER_NAME  = "chrome://navigator/content/navigator.xul";
    var NODE_NAME       = "chrome://"+myToolbarId+"/content/"+myToolbarId+"Overlay.xul";

    var dirServ = Components.classes['@mozilla.org/file/directory_service;1'].createInstance();
    dirServ = dirServ.QueryInterface(Components.interfaces.nsIProperties);

    var appChrome = dirServ.get("AChrom", Components.interfaces.nsIFile);
    var userChrome = dirServ.get("UChrm", Components.interfaces.nsIFile);

    var processDir = appChrome.clone();
    processDir.append("overlays.rdf");
    if (!processDir.exists())
    {
	processDir = appChrome.clone();
	processDir.append("overlayinfo");
	processDir.append("navigator");
	processDir.append("content");
	processDir.append("overlays.rdf");
    } 
  
    removeRDFContainer(processDir, CONTAINER_NAME, NODE_NAME);

// Now try to remove it from the user chrome 
    processDir = userChrome.clone();

    processDir.append("overlays.rdf");
    removeRDFContainer(processDir, CONTAINER_NAME, NODE_NAME);

    // remove chrome.rdf entries
  
    CONTAINER_NAME = 'urn:mozilla:package:root';
    NODE_NAME = 'urn:mozilla:package:yahootoolbar';
  
    processDir = appChrome.clone();
    processDir.append("chrome.rdf");
  
    removeRDFContainer(processDir, CONTAINER_NAME, NODE_NAME);

// Now try to clean up the user chrome
    processDir = userChrome.clone();
    processDir.append("chrome.rdf");
  
    removeRDFContainer(processDir, CONTAINER_NAME, NODE_NAME);
  
    // remove the installed-chrome.txt entries - thanks to HJ for the code
    processDir = appChrome.clone();
    processDir.append("installed-chrome.txt");
    
    var entries = ReadFrom(processDir);
    entries = entries.replace(new RegExp('[^\\n\\r]+'+"resource:/chrome/"+myToolbarId+".jar!/"+'.*[\\n\\r]+', 'g'), '');
    WriteTo(processDir, entries); 
    

    //remove the .jar file
    processDir = appChrome.clone();
    processDir.append("yahootoolbar.jar");
    //(false) because we don't want to recurse
    if (processDir.exists()) 
        processDir.remove(false);

    //remove the .jar file
    processDir = userChrome.clone();
    processDir.append("yahootoolbar.jar");
    //(false) because we don't want to recurse
    if (processDir.exists()) 
       processDir.remove(false);
  

    // ?? why is this nested here??
    function removeRDFContainer(fileRef, containerName, nodeName)
    {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect"); 
        var C                       = Components;
        const RDF_SRVC_PROG_ID      = '@mozilla.org/rdf/rdf-service;1';
        const RDF_CONTAINER_PROG_ID = '@mozilla.org/rdf/container;1';
        const RDF_DS_PROG_ID        = '@mozilla.org/rdf/datasource;1?name=xml-datasource';
        
        if (!fileRef.exists()) 
            return;
     
        var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    
        // Interface change - ns7 (mozilla 1.0) needs nsIIOService, latter (mozilla 1.2) needs nsIFileProtocolHandler
        if(Components.interfaces.nsIFileProtocolHandler){
            var fileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);  
            var myFileUrl = fileHandler.getURLSpecFromFile(fileRef);
        } else {
            var myFileUrl = ioService.getURLSpecFromFile(fileRef);    
        }  
      
        try {
            var rdf   = C.classes
                      [RDF_SRVC_PROG_ID].getService(C.interfaces.nsIRDFService);
    
            var ds    = rdf.GetDataSource(myFileUrl);
      
            var rem   = ds.QueryInterface(C.interfaces.nsIRDFRemoteDataSource);
            if (!rem.loaded) {
                rdf.UnregisterDataSource(ds);
                ds        = C.classes[RDF_DS_PROG_ID].
                        getService(C.interfaces.nsIRDFDataSource);
                rem       = ds.QueryInterface(C.interfaces.nsIRDFRemoteDataSource);
                rem.Init(myFileUrl);
            }
            rem.Refresh(true)
            var rs  = rdf.GetResource(containerName);
            var container = C.classes[RDF_CONTAINER_PROG_ID].
                    getService(C.interfaces.nsIRDFContainer);
            container.Init(rem, rs);
    
            var el = rdf.GetLiteral(nodeName); // remove if it's a literal
            container.RemoveElement(el, true);
            el = rdf.GetResource(nodeName); // remove it it's a resource
            container.RemoveElement(el, true);
            rem.Flush()
            rem.Refresh(true)
        } catch (e) {
            return;
        }
    }
    // end removeRDFContainer
  
    function ReadFrom(aFile) 
    {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect"); 
        var stream = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream);
        stream.init(aFile, 1, 0, false); // open as "read only"
    
        var scriptableStream = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
        scriptableStream.init(stream);
    
        var fileSize = scriptableStream.available();
        var fileContents = scriptableStream.read(fileSize);
    
        scriptableStream.close();
        stream.close();
    
        return fileContents;
    }
    
    function WriteTo(aFile, aContent) 
    {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect"); 
        if (aFile.exists())
        aFile.remove(true);
        aFile.create(aFile.NORMAL_FILE_TYPE, 0766);

        var stream = Components.classes['@mozilla.org/network/file-output-stream;1'].createInstance(Components.interfaces.nsIFileOutputStream);
        stream.init(aFile, 2, 0666, false); // open as "write only"

        stream.write(aContent, aContent.length);

        stream.close();
    }
}
// end Uninstall
